import { toSingularAction } from './singular';
import { nodeToType } from './structure';
import { NodeType } from './types/types';
export const generateSingle = (nodeName) => (state = {}, action) => {
    switch (action.type) {
        case `SET_${nodeName.toUpperCase()}`:
            return action.payload;
        case `PATCH_${nodeName.toUpperCase()}`:
            return { ...state, ...action.payload };
        default:
            return state || {};
    }
};
export const generateQueue = (nodeName) => (state = [], action) => {
    let newState;
    switch (action.type) {
        case `SET_${nodeName.toUpperCase()}`:
            return action.payload;
        case `PUSH_${toSingularAction(nodeName)}`:
            return [...state, action.payload];
        case `POP_${toSingularAction(nodeName)}`:
            newState = [...state];
            newState.pop();
            return newState;
        case `CLEAR_${nodeName.toUpperCase()}`:
            return [];
        case `PUSH_MANY_${nodeName.toUpperCase()}`:
            const { items = [] } = action.payload || {};
            return [...state, ...(items || [])];
        default:
            return state || [];
    }
};
export const generateCollection = (nodeName) => {
    const item = (state = {}, action) => {
        switch (action.type) {
            case `SET_${toSingularAction(nodeName)}`:
                return action.payload;
            case `PATCH_${toSingularAction(nodeName)}`:
                return {
                    ...state,
                    ...action.payload,
                };
            default:
                return state;
        }
    };
    const items = (state = {}, action) => {
        let newState;
        switch (action.type) {
            case `SET_${nodeName.toUpperCase()}`:
                return action.payload;
            case `SET_${toSingularAction(nodeName)}`:
            case `PATCH_${toSingularAction(nodeName)}`:
                if (!action.payload || !action.id) {
                    return state;
                }
                return {
                    ...state,
                    [action.id]: item(state[action.id], action),
                };
            case `DELETE_${toSingularAction(nodeName)}`:
                if (!action.payload || !action.id) {
                    return state;
                }
                newState = { ...state };
                delete newState[action.id];
                return newState;
            case `SET_MANY_${nodeName.toUpperCase()}`:
                return {
                    ...state,
                    ...action.payload,
                };
            default:
                return state || {};
        }
    };
    return items;
};
export const generateGroupedList = (nodeName) => {
    const listItems = (state, action) => {
        let newState;
        const { items = [] } = action.payload || {};
        switch (action.type) {
            case `SET_${nodeName.toUpperCase()}_ITEMS`:
                return items;
            case `PUSH_${nodeName.toUpperCase()}_ITEM`:
            case `PUSH_MANY_${nodeName.toUpperCase()}_ITEMS`:
                return [...state, ...items];
            case `POP_${nodeName.toUpperCase()}_ITEM`:
                newState = [...state];
                newState.pop();
                return newState;
            case `CLEAR_${nodeName.toUpperCase()}_ITEMS`:
                return [];
            default:
                return state || [];
        }
    };
    const item = (state = {}, action) => {
        switch (action.type) {
            case `SET_${toSingularAction(nodeName)}`:
                return action.payload;
            case `PATCH_${toSingularAction(nodeName)}`:
                return {
                    ...state,
                    ...action.payload,
                };
            case `SET_${nodeName.toUpperCase()}_ITEMS`:
            case `PUSH_${nodeName.toUpperCase()}_ITEM`:
            case `POP_${nodeName.toUpperCase()}_ITEM`:
            case `CLEAR_${nodeName.toUpperCase()}_ITEMS`:
            case `PUSH_MANY_${nodeName.toUpperCase()}_ITEMS`:
                return {
                    ...state,
                    items: listItems(state.items, action),
                };
            default:
                return state;
        }
    };
    const items = (state = {}, action) => {
        let newState;
        switch (action.type) {
            case `SET_${nodeName.toUpperCase()}`:
                return action.payload;
            case `SET_${toSingularAction(nodeName)}`:
            case `PATCH_${toSingularAction(nodeName)}`:
            case `SET_${nodeName.toUpperCase()}_ITEMS`:
            case `PUSH_${nodeName.toUpperCase()}_ITEM`:
            case `POP_${nodeName.toUpperCase()}_ITEM`:
            case `CLEAR_${nodeName.toUpperCase()}_ITEMS`:
            case `PUSH_MANY_${nodeName.toUpperCase()}_ITEMS`:
                if (!action.payload || !action.id) {
                    return state;
                }
                return {
                    ...state,
                    [action.id]: item(state[action.id], action),
                };
            case `DELETE_${toSingularAction(nodeName)}`:
                if (!action.payload || !action.id) {
                    return state;
                }
                newState = { ...state };
                delete newState[action.id];
                return newState;
            case `SET_MANY_${nodeName.toUpperCase()}`:
                return {
                    ...state,
                    ...action.payload,
                };
            default:
                return state || {};
        }
    };
    return items;
};
export const generateReducersForStore = (storeState) => {
    let output = {};
    const keys = Object.keys(storeState);
    for (const key of keys) {
        const type = nodeToType(storeState[key]);
        switch (type) {
            case NodeType.SINGLE_NODE:
                output[key] = generateSingle(String(key));
                break;
            case NodeType.QUEUE_NODE:
                output[key] = generateQueue(String(key));
                break;
            case NodeType.COLLECTION_NODE:
                output[key] = generateCollection(String(key));
                break;
            case NodeType.GROUPED_LIST_NODE:
                output[key] = generateGroupedList(String(key));
                break;
        }
    }
    return output;
};
